import pytest

from depot.manager import DepotManager


@pytest.yield_fixture(scope='function', autouse=True)
def depot(temporary_directory):
    DepotManager.configure('default', {
        'depot.backend': 'depot.io.local.LocalFileStorage',
        'depot.storage_path': temporary_directory
    })

    yield DepotManager.get()

    DepotManager._clear()



from onegov.chat import MessageCollection


def test_collection_filter(session):
    msgs = MessageCollection(session)
    msgs.add(channel_id='public', text='Yo!')
    msgs.add(channel_id='public', text='Yo Sup?')
    msgs.add(channel_id='public', text='Sup?')
    msgs.add(channel_id='private', text='U female?')

    public = MessageCollection(session, channel_id='public')
    assert public.query().count() == 3

    private = MessageCollection(session, channel_id='private')
    assert private.query().count() == 1

    msgs.newer_than = msgs.latest_message().id
    latest = msgs.add(channel_id='private', text='Nope')
    assert msgs.query().count() == 1

    msgs.newer_than = None
    msgs.older_than = latest.id
    assert msgs.query().count() == 4

    msgs.limit = 1
    assert msgs.query().count() == 1



from onegov.chat.models import Message
from onegov.core.collection import GenericCollection
from sqlalchemy import desc


class MessageCollection(GenericCollection):

    def __init__(self, session, type='*', channel_id='*', newer_than=None,
                 older_than=None, limit=None, load='older-first'):
        super().__init__(session)
        self.type = type
        self.channel_id = channel_id
        self.newer_than = newer_than
        self.older_than = older_than
        self.limit = limit
        self.load = load

        assert self.load in ('older-first', 'newer-first')

    @property
    def model_class(self):
        return Message.get_polymorphic_class(self.type, Message)

    def add(self, **kwargs):
        if self.type != '*':
            kwargs.setdefault('type', self.type)
        return super().add(**kwargs)

    def query(self):
        """ Queries the messages with the given parameters. """

        q = self.session.query(self.model_class)

        if self.type != '*':
            q = q.filter_by(type=self.type)

        if self.channel_id != '*':
            q = q.filter_by(channel_id=self.channel_id)

        if self.newer_than is not None:
            q = q.filter(self.model_class.id > self.newer_than)

        if self.older_than is not None:
            q = q.filter(self.model_class.id < self.older_than)

        if self.load == 'older-first':
            q = q.order_by(self.model_class.id)
        else:
            q = q.order_by(desc(self.model_class.id))

        if self.limit is not None:
            q = q.limit(self.limit)

        return q

    def latest_message(self, offset=0):
        """ Returns the latest messages in descending order (newest first). """
        q = self.session.query(self.model_class)
        q = q.order_by(desc(self.model_class.id))

        if offset:
            q = q.offset(min(offset, q.count()))

        return q.first()
